@@ -1,11 +1,14 @@
 import asyncio
+import atexit
 import logging
 import os
 import signal
 import subprocess
 import sys
+from contextlib import AsyncExitStack
 from contextlib import suppress
 from dataclasses import dataclass
+from functools import partial
 from functools import wraps
 from os import listdir
 from os.path import dirname
@@ -48,6 +51,7 @@ from dipdup.migrations import deprecated_handlers
 from dipdup.models import Index
 from dipdup.models import Schema
 from dipdup.utils import iter_files
+from dipdup.utils.database import execute_sql_scripts
 from dipdup.utils.database import set_decimal_context
 from dipdup.utils.database import tortoise_wrapper
 from dipdup.utils.database import wipe_schema
@@ -80,15 +84,13 @@ def cli_wrapper(fn):
         loop = asyncio.get_running_loop()
         loop.add_signal_handler(signal.SIGINT, lambda: asyncio.ensure_future(shutdown()))
         try:
-            with DipDupError.wrap():
-                await fn(*args, **kwargs)
+            await fn(*args, **kwargs)
         except (KeyboardInterrupt, asyncio.CancelledError):
             pass
-        except DipDupError as e:
-            # FIXME: No traceback in test logs
-            _logger.critical(e.__repr__())
-            _logger.info(e.format())
-            quit(1)
+        except Exception as e:
+            help_message = e.format() if isinstance(e, DipDupError) else DipDupError().format()
+            atexit.register(partial(click.echo, help_message, err=True))
+            raise
 
     return wrapper
 
@@ -303,9 +305,10 @@ async def hasura(ctx):
 
 
 @hasura.command(name='configure', help='Configure Hasura GraphQL Engine')
+@click.option('--force', is_flag=True, help='Proceed even if Hasura is already configured')
 @click.pass_context
 @cli_wrapper
-async def hasura_configure(ctx):
+async def hasura_configure(ctx, force: bool):
     config: DipDupConfig = ctx.obj.config
     url = config.database.connection_string
     models = f'{config.package}.models'
@@ -319,7 +322,7 @@ async def hasura_configure(ctx):
 
     async with tortoise_wrapper(url, models):
         async with hasura_gateway:
-            await hasura_gateway.configure()
+            await hasura_gateway.configure(force)
 
 
 @cli.group(help='Manage database schema')
@@ -389,6 +392,30 @@ async def schema_wipe(ctx, immune: bool):
     _logger.info('Schema wiped')
 
 
+@schema.command(name='init', help='Initialize database schema')
+@click.pass_context
+@cli_wrapper
+async def schema_init(ctx):
+    config: DipDupConfig = ctx.obj.config
+    url = config.database.connection_string
+    dipdup = DipDup(config)
+
+    _logger.info('Initializing schema `%s`', url)
+
+    async with AsyncExitStack() as stack:
+        await dipdup._set_up_database(stack)
+        await dipdup._set_up_hooks()
+        await dipdup._create_datasources()
+        await dipdup._initialize_schema()
+
+        # NOTE: It's not necessary a reindex, but it's safe to execute built-in scripts to (re)create views.
+        conn = get_connection(None)
+        sql_path = join(dirname(__file__), 'sql', 'on_reindex')
+        await execute_sql_scripts(conn, sql_path)
+
+    _logger.info('Schema initialized')
+
+
 @schema.command(name='export', help='Print schema SQL including `on_reindex` hook')
 @click.pass_context
 @cli_wrapper
@@ -400,6 +427,11 @@ async def schema_export(ctx):
     async with tortoise_wrapper(url, models):
         conn = get_connection(None)
         output = get_schema_sql(conn, False) + '\n'
-        for file in iter_files(join(config.package_path, 'sql', 'on_reindex')):
-            output += file.read() + '\n'
+        dipdup_sql_path = join(dirname(__file__), 'sql', 'on_reindex')
+        project_sql_path = join(config.package_path, 'sql', 'on_reindex')
+
+        for sql_path in (dipdup_sql_path, project_sql_path):
+            for file in iter_files(sql_path):
+                output += file.read() + '\n'
+
         echo(output)
