@@ -11,7 +11,6 @@ from collections import defaultdict
 from contextlib import suppress
 from copy import copy
 from dataclasses import field
-from enum import Enum
 from functools import cached_property
 from os import environ as env
 from os.path import dirname
@@ -39,12 +38,15 @@ from pydantic.json import pydantic_encoder
 from ruamel.yaml import YAML
 from typing_extensions import Literal
 
+from dipdup.datasources.metadata.enums import MetadataNetwork
 from dipdup.datasources.subscription import BigMapSubscription
 from dipdup.datasources.subscription import OriginationSubscription
 from dipdup.datasources.subscription import Subscription
 from dipdup.datasources.subscription import TransactionSubscription
+from dipdup.enums import OperationType
 from dipdup.enums import ReindexingAction
 from dipdup.enums import ReindexingReasonC
+from dipdup.enums import SkipHistory
 from dipdup.exceptions import ConfigInitializationException
 from dipdup.exceptions import ConfigurationError
 from dipdup.utils import import_from
@@ -54,16 +56,11 @@ from dipdup.utils import snake_to_pascal
 ENV_VARIABLE_REGEX = r'\${([\w]*):-(.*)}'
 DEFAULT_RETRY_COUNT = 3
 DEFAULT_RETRY_SLEEP = 1
+DEFAULT_METADATA_URL = 'https://metadata.dipdup.net'
 
 _logger = logging.getLogger('dipdup.config')
 
 
-class OperationType(Enum):
-    transaction = 'transaction'
-    origination = 'origination'
-    migration = 'migration'
-
-
 @dataclass
 class SqliteDatabaseConfig:
     """
@@ -210,7 +207,7 @@ class TzktDatasourceConfig(NameMixin):
     http: Optional[HTTPConfig] = None
 
     def __hash__(self):
-        return hash(self.url)
+        return hash(self.kind + self.url)
 
     def __post_init_post_parse__(self) -> None:
         super().__post_init_post_parse__()
@@ -240,7 +237,7 @@ class BcdDatasourceConfig(NameMixin):
     http: Optional[HTTPConfig] = None
 
     def __hash__(self):
-        return hash(self.url + self.network)
+        return hash(self.kind + self.url + self.network)
 
     @validator('url', allow_reuse=True)
     def valid_url(cls, v):
@@ -271,7 +268,23 @@ class CoinbaseDatasourceConfig(NameMixin):
         return hash(self.kind)
 
 
-DatasourceConfigT = Union[TzktDatasourceConfig, BcdDatasourceConfig, CoinbaseDatasourceConfig]
+@dataclass
+class MetadataDatasourceConfig(NameMixin):
+    kind: Literal['metadata']
+    network: MetadataNetwork
+    url: str = DEFAULT_METADATA_URL
+    http: Optional[HTTPConfig] = None
+
+    def __hash__(self):
+        return hash(self.kind + self.url + self.network.value)
+
+
+DatasourceConfigT = Union[
+    TzktDatasourceConfig,
+    BcdDatasourceConfig,
+    CoinbaseDatasourceConfig,
+    MetadataDatasourceConfig,
+]
 
 
 @dataclass
@@ -564,8 +577,8 @@ class CallbackMixin(CodegenMixin):
 
     def __post_init_post_parse__(self):
         self._callback_fn = None
-        if self.callback and self.callback != pascal_to_snake(self.callback):
-            raise ConfigurationError('`callback` field must conform to snake_case naming style')
+        if self.callback and self.callback != pascal_to_snake(self.callback, strip_dots=False):
+            raise ConfigurationError('`callback` field must be a valid Python module name')
 
     @cached_property
     def kind(self) -> str:
@@ -667,6 +680,8 @@ class IndexConfig(TemplateValuesMixin, NameMixin, SubscriptionsMixin, ParentMixi
 
         # NOTE: We need to preserve datasource URL but remove it's HTTP tunables to avoid false-positives.
         config_dict['datasource'].pop('http', None)
+        # NOTE: Same for BigMapIndex tunables
+        config_dict.pop('skip_history', None)
 
         config_json = json.dumps(config_dict)
         return hashlib.sha256(config_json.encode()).hexdigest()
@@ -804,6 +819,8 @@ class BigMapIndexConfig(IndexConfig):
     datasource: Union[str, TzktDatasourceConfig]
     handlers: Tuple[BigMapHandlerConfig, ...]
 
+    skip_history: SkipHistory = SkipHistory.never
+
     first_level: int = 0
     last_level: int = 0
 
@@ -855,11 +872,6 @@ class HasuraConfig:
             raise ConfigurationError(f'`{v}` is not a valid Hasura URL')
         return v.rstrip('/')
 
-    @validator('source', allow_reuse=True)
-    def valid_source(cls, v):
-        if v != 'default':
-            raise NotImplementedError('Multiple Hasura sources are not supported at the moment')
-
     @cached_property
     def headers(self) -> Dict[str, str]:
         if self.admin_secret:
@@ -1024,7 +1036,6 @@ class DipDupConfig:
         paths: List[str],
         environment: bool = True,
     ) -> 'DipDupConfig':
-
         current_workdir = os.path.join(os.getcwd())
 
         json_config: Dict[str, Any] = {}
@@ -1033,8 +1044,11 @@ class DipDupConfig:
             path = os.path.join(current_workdir, path)
 
             _logger.debug('Loading config from %s', path)
-            with open(path) as file:
-                raw_config = file.read()
+            try:
+                with open(path) as file:
+                    raw_config = file.read()
+            except OSError as e:
+                raise ConfigurationError(str(e))
 
             if environment:
                 _logger.debug('Substituting environment variables')
