@@ -29,6 +29,7 @@ from dipdup.config import HasuraConfig
 from dipdup.config import HTTPConfig
 from dipdup.config import PostgresDatabaseConfig
 from dipdup.exceptions import ConfigurationError
+from dipdup.exceptions import HasuraError
 from dipdup.http import HTTPGateway
 from dipdup.models import Schema
 from dipdup.utils import iter_files
@@ -77,10 +78,6 @@ class Field:
         return humps.decamelize(self.name)
 
 
-class HasuraError(RuntimeError):
-    ...
-
-
 class HasuraGateway(HTTPGateway):
     _default_http_config = HTTPConfig(
         cache=False,
@@ -102,7 +99,7 @@ class HasuraGateway(HTTPGateway):
         self._hasura_config = hasura_config
         self._database_config = database_config
 
-    async def configure(self) -> None:
+    async def configure(self, force: bool = False) -> None:
         """Generate Hasura metadata and apply to instance with credentials from `hasura` config section."""
 
         if self._database_config.schema_name != 'public':
@@ -116,25 +113,25 @@ class HasuraGateway(HTTPGateway):
         metadata = await self._fetch_metadata()
         metadata_hash = self._hash_metadata(metadata)
 
-        if hasura_schema.hash == metadata_hash:
+        if not force and hasura_schema.hash == metadata_hash:
             self._logger.info('Metadata is up to date, no action required')
             return
 
-        await self._reset_metadata()
-        metadata = await self._fetch_metadata()
-
-        # NOTE: Hasura metadata updated in three steps.
-        # NOTE: Order matters because queries must be generated after applying table customization to be valid.
-        # NOTE: 1. Generate and apply tables metadata.
-        source_tables_metadata = await self._generate_source_tables_metadata()
-        metadata['sources'][0]['tables'] = source_tables_metadata
+        # NOTE: Find chosen source and overwrite its tables
+        source_name = self._hasura_config.source
+        for source in metadata['sources']:
+            if source['name'] == source_name:
+                source['tables'] = await self._generate_source_tables_metadata()
+                break
+        else:
+            raise HasuraError(f'Source `{source_name}` not found in metadata')
         await self._replace_metadata(metadata)
 
-        # NOTE: 2. Apply table customization and refresh metadata
+        # NOTE: Apply table customizations before generating queries
         await self._apply_table_customization()
         metadata = await self._fetch_metadata()
 
-        # NOTE: 3. Generate and apply queries and rest endpoints
+        # NOTE: Generate and apply queries and REST endpoints
         query_collections_metadata = await self._generate_query_collections_metadata()
         self._logger.info('Adding %s generated and user-defined queries', len(query_collections_metadata))
         metadata['query_collections'] = [
@@ -152,6 +149,7 @@ class HasuraGateway(HTTPGateway):
 
         await self._replace_metadata(metadata)
 
+        # NOTE: Fetch metadata once again (to do: find out why is it necessary) and save its hash for future comparisons
         metadata = await self._fetch_metadata()
         metadata_hash = self._hash_metadata(metadata)
         hasura_schema.hash = metadata_hash  # type: ignore
@@ -170,7 +168,7 @@ class HasuraGateway(HTTPGateway):
         )
         self._logger.debug('Response: %s', result)
         if 'error' in result or 'errors' in result:
-            raise HasuraError('Can\'t configure Hasura instance', result)
+            raise HasuraError(result)
         return result
 
     async def _healthcheck(self) -> None:
@@ -183,7 +181,7 @@ class HasuraGateway(HTTPGateway):
                     break
             await asyncio.sleep(1)
         else:
-            raise HasuraError(f'Hasura instance not responding for {timeout} seconds')
+            raise HasuraError(f'Not responding for {timeout} seconds')
 
         version_json = await (
             await self._http._session.get(
@@ -192,7 +190,7 @@ class HasuraGateway(HTTPGateway):
         ).json()
         version = version_json['version']
         if version.startswith('v1'):
-            raise HasuraError('Hasura v1 is not supported.')
+            raise HasuraError('v1 is not supported, upgrade to the latest stable version.')
 
         self._logger.info('Connected to Hasura %s', version)
 
@@ -273,7 +271,7 @@ class HasuraGateway(HTTPGateway):
             for field in model._meta.fields_map.values():
                 if isinstance(field, fields.relational.ForeignKeyFieldInstance):
                     if not isinstance(field.related_name, str):
-                        raise HasuraError(f'`related_name` of `{field}` must be set')
+                        raise HasuraError(f'`related_name` of `{field}` field must be set')
 
                     related_table_name = model_tables[field.model_name]
                     field_name = field.model_field_name
@@ -293,7 +291,7 @@ class HasuraGateway(HTTPGateway):
 
                 elif isinstance(field, fields.relational.ManyToManyFieldInstance):
                     if not isinstance(field.related_name, str):
-                        raise HasuraError(f'`related_name` of `{field}` must be set')
+                        raise HasuraError(f'`related_name` of `{field}` field must be set')
 
                     related_table_name = model_tables[field.model_name]
                     junction_table_name = field.through
@@ -342,6 +340,9 @@ class HasuraGateway(HTTPGateway):
         for query_name, query in self._iterate_graphql_queries():
             queries.append({'name': query_name, 'query': query})
 
+        # NOTE: This is the only view we add by ourselves and thus know all params. Won't work for any view.
+        queries.append(self._format_rest_head_status_query())
+
         return queries
 
     async def _generate_rest_endpoints_metadata(self, query_names: List[str]) -> List[Dict[str, Any]]:
@@ -455,6 +456,16 @@ class HasuraGateway(HTTPGateway):
             'query': 'query ' + name + ' (' + query_arg + ') {' + table + '(' + query_filter + ') {' + query_fields + '}}',
         }
 
+    def _format_rest_head_status_query(self) -> Dict[str, Any]:
+        name = 'dipdup_head_status'
+        if self._hasura_config.camel_case:
+            name = humps.camelize(name)
+
+        return {
+            'name': name,
+            'query': 'query ' + name + ' ($name: String!) {' + name + '(where: {name: {_eq: $name}}) {status}}',
+        }
+
     def _format_rest_endpoint(self, query_name: str) -> Dict[str, Any]:
         return {
             "definition": {
