@@ -64,9 +64,11 @@ def snake_to_pascal(value: str) -> str:
     return humps.pascalize(value.replace('.', '_'))
 
 
-def pascal_to_snake(value: str) -> str:
+def pascal_to_snake(value: str, strip_dots: bool = True) -> str:
     """humps wrapper for Python imports"""
-    return humps.depascalize(value.replace('.', '_')).replace('__', '_')
+    if strip_dots:
+        value = value.replace('.', '_')
+    return humps.depascalize(value).replace('__', '_')
 
 
 def split_by_chunks(input_: List[Any], size: int) -> Iterator[List[Any]]:
